using UnityEngine;
using System.Collections;

public class Controller : MonoBehaviour {

	public TotemMatcherMaker.Totems identifier;

	public float speed = 0.4f;
	public float jumpForce = 10f;

	public float meleeCooldown = 0.5f;
	public float nextMelee = 0f;

	public float rangedCooldown = 0.5f;
	public float nextRanged = 0f;

	public string control = "K";

	public Controller topTotem, bottomTotem;

	public GameObject legs;

	public Rigidbody2D rb2d;

	public GameObject projectile;
	public GameObject partProjectile;

	public TotemStats ts;

	public float totemHeightOffset;
	//public Vector2 totemBoundsOffset;
	public float shootForceY = 0;
	public float shootForceX = 1;

	public ArrowTotemHelper arrowTotemHelper;

	public Animator anim;

	public AudioSource audiosrc;
	public AudioClip InTot;
	public AudioClip OutTot;

	public AudioClip shot;

	void OnValidate(){
		if(!rb2d){
			rb2d = GetComponent<Rigidbody2D>();
		}
		if(!anim){
			anim = GetComponent<Animator>();
		}
		arrowTotemHelper = GetComponent<ArrowTotemHelper>();
		ts = gameObject.GetComponent<TotemStats>();
		Transform legsTransform = transform.Find("legs");
		if(legsTransform)
			legs = legsTransform.gameObject;
		//totemBoundsOffset = GetComponent<BoxCollider2D>().offset;
		totemHeightOffset = GetComponent<BoxCollider2D>().bounds.size.y;
		audiosrc = gameObject.GetComponent<AudioSource>();

	}

	void OnDrawGizmos()
	{
		Gizmos.color = Color.black;
		Gizmos.DrawWireSphere(transform.position, 2);
	}

	void Update ()
	{
		ts.energy += (int)(0.2f * Time.deltaTime);
		
		if(GameManager.Singleton.freezeGlobalMovement) return;
		if(Input.GetButtonDown(control + "Y"))
		{
			if(bottomTotem){
				DecloupeTotem();
			} else {
				Collider2D[] targets = Physics2D.OverlapCircleAll(transform.position, 2, 1 << 9);
				foreach(Collider2D target in targets){
					Controller controllerTarget = target.GetComponent<Controller>();
					if(controllerTarget == this || controllerTarget == topTotem) continue;
					controllerTarget.JoinTotem(this);
					break;
				}
			}
		}
		
		if(Input.GetButtonDown(control + "X") && Time.time > nextMelee){
			//MeleeAttack();
			anim.SetTrigger("Bash");
			nextMelee = Time.time + meleeCooldown;
		}
		
		if(Input.GetButtonDown(control + "B") && Time.time > nextRanged){
			//RangedAttack();
			Shoot();
			nextRanged = Time.time + rangedCooldown;
		}

		if(Input.GetAxis(control + "Hor") < 0)
		{
			
			if(bottomTotem)
				transform.localScale = new Vector3(-1*transform.parent.lossyScale.x, 1, 1);
			else
				transform.localScale = new Vector3(-1, 1, 1);

		}
		else if(Input.GetAxis(control + "Hor") > 0)
		{
			if(bottomTotem)
				transform.localScale = new Vector3(1*transform.parent.lossyScale.x, 1, 1);
			else
				transform.localScale = new Vector3(1, 1, 1);
		}

		if(bottomTotem) return;

		transform.position += speed * Input.GetAxis(control + "Hor") * transform.right;

		if(Physics2D.Raycast(transform.position, Vector2.down, 1.55f, 1 << 14)){
			if(identifier != TotemMatcherMaker.Totems.Tartaruga){
				return;
			}

			if(Input.GetButton(control + "A"))
			{
				if(rb2d.velocity.y < 0.1f)
				{
					rb2d.AddForce(jumpForce * transform.up*1.5f, ForceMode2D.Impulse);
				}
			}

			return;

		}

		if(Input.GetButton(control + "A"))
		{
			if(rb2d.velocity.y < 0.1f)
			{
				if(Physics2D.Raycast(transform.position, Vector2.down, 1.55f, 1 << 8))
				{
					rb2d.AddForce(jumpForce * transform.up, ForceMode2D.Impulse);
				}
			}
		}

	}

	void Shoot()
	{
		if(ts.energy > 0)
		{
			GameObject proj = (GameObject)GameObject.Instantiate(projectile, this.transform.position, transform.rotation);
			proj.GetComponent<Rigidbody2D>().velocity = new Vector2(shootForceX * 50 * transform.lossyScale.x, shootForceY);
			ts.energy -= 15;
			audiosrc.PlayOneShot(shot);
		}
	}

	void JoinTotem (Controller totem)
	{
		audiosrc.PlayOneShot(InTot);
		if(topTotem)
		{
			topTotem.GetComponent<Controller>().JoinTotem(totem);
		}
		else
		{
			topTotem = totem;
			totem.bottomTotem = this;
			totem.gameObject.transform.parent = transform;
			TotemJoiner totemJoiner = totem.gameObject.AddComponent<TotemJoiner>();
			totemJoiner.targetPosition = (Vector3.up * (totem.totemHeightOffset + totemHeightOffset)/2f);
			//totem.gameObject.transform.localPosition = (Vector3.up * (totem.totemHeightOffset + totemHeightOffset)/2f) /*+ (Vector3)totemBoundsOffset*/;
			if(totem.legs) totem.legs.SetActive(false);
			arrowTotemHelper.arrow.SetActive(false);
			totem.arrowTotemHelper.arrow.SetActive(false);
			totem.rb2d.isKinematic = true;
		}

	}

	void DecloupeTotem ()
	{
		audiosrc.PlayOneShot(OutTot);

		if(!bottomTotem) return;

		bottomTotem.topTotem = null;
		rb2d.isKinematic = false;
		transform.parent = null;
		if(legs) legs.SetActive(true);
		bottomTotem = null;
		rb2d.AddForce(jumpForce * transform.up, ForceMode2D.Impulse);
	}

	void MeleeTestHit(){
		RaycastHit2D[] hitteds = Physics2D.RaycastAll(transform.position, transform.right*transform.lossyScale.x, 2f, 1 << 12);
			foreach(RaycastHit2D hitted in hitteds){
			ITakeDamage stats = hitted.transform.GetComponent<ITakeDamage>();
			if(stats != null){
				stats.takeDmg(2);
			}
		}
	}
}
