﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class FinalBossBehaviour : MonoBehaviour, ITakeDamage {

	public GameObject gem, laserTarget, fireball, indio;
	public Vector3 lastTargetPos, initialPos;
	public LineRenderer myLineRenderer;
	public float laserAcquisitionRadius, laserOnTargetRadius, laserHeat, speed, direction, nextFireball, fireballDelay, fireballSpeed, fireballCounter, nextIndio, indioDelay, indioCounter;
	public int life;
	public bool canAct = false, descending = false, spawning = false, ascending = false;
	public Animator anim;
	public Collider2D myCollider;
	public Image hb;

	void Start(){
		myLineRenderer.sortingOrder = 4;
		hb = GameObject.Find("BARRAVIDA").GetComponent<Image>();
	}
	
	void setBars()
	{
		hb.fillAmount = life/200f;
	}
	
	public void death()
	{
		Application.LoadLevel(5);
	}
	
	void Update () 
	{
		
		if(life <= 0)
		{
			death();
		}
		
		setBars();

		if(!canAct){
			if(Vector2.Distance(transform.position, initialPos) < 0.25f){
				anim.SetTrigger("Start");
				return;
			}
			transform.position += (initialPos - transform.position).normalized * 0.25f;
			return;
		}

		if(descending){
			transform.position -= transform.up * speed/5f;
			if(Physics2D.Raycast(transform.position, -transform.up, 5f, 1 << 8)){
				spawning = true;
				descending = false;
			}
			return;
		}

		if(ascending){
			if(Vector2.Distance(transform.position, initialPos) < 0.25f){
				ascending = false;
				return;
			}
			transform.position += (initialPos - transform.position).normalized * 0.5f;

			return;
		}

		if(spawning){
			if(Time.time > nextIndio){
				nextIndio = Time.time + indioDelay;
				GameObject.Instantiate(indio, transform.position, Quaternion.identity);
				if(++indioCounter >= 3){
					spawning = false;
					ascending = true;
					indioCounter = 0;
				}
			}
			return;
		}


		if(transform.position.x > 70f || transform.position.x < 25f){
			direction *= -1f;
		}

		transform.position += transform.right * speed * direction;

		if(laserTarget){
			laserHeat += Time.deltaTime;
			if(laserHeat > 1f){
				laserHeat = 1f;
				if(Time.time > nextFireball){
					nextFireball = Time.time + fireballDelay;
					GameObject fb = GameObject.Instantiate(fireball, gem.transform.position, Quaternion.identity) as GameObject;
					fb.GetComponent<Rigidbody2D>().velocity = (laserTarget.transform.position - gem.transform.position).normalized * fireballSpeed;
					if(++fireballCounter >= 5) {
						fireballCounter = 0;
						descending = true;
					}
				}
			}
			
			if(Vector2.Distance(laserTarget.transform.position, gem.transform.position) > laserOnTargetRadius){
				lastTargetPos = laserTarget.transform.position;
				laserTarget = null;
			} else {
				myLineRenderer.SetPosition(0, gem.transform.position);
				myLineRenderer.SetPosition(1, Vector3.Lerp(gem.transform.position, laserTarget.transform.position, laserHeat));
				myLineRenderer.SetWidth(0.1f, laserHeat);
			}

		} else {
			if(laserHeat > 0f){
				laserHeat -= Time.deltaTime*1.5f;
				if(laserHeat < 0f){
					laserHeat = 0f;
					myLineRenderer.enabled = false;
				}
				myLineRenderer.SetPosition(0, gem.transform.position);
				myLineRenderer.SetPosition(1, Vector3.Lerp(gem.transform.position, lastTargetPos, laserHeat));
				myLineRenderer.SetWidth(0.1f, laserHeat);

			} else {
				Collider2D collider = Physics2D.OverlapCircle(gem.transform.position, laserAcquisitionRadius, 1 << 9);
				if(collider){
					laserHeat = 0f;
					laserTarget = collider.gameObject;
					//myLineRenderer.enabled = true;
					myLineRenderer.SetPosition(0, gem.transform.position);
					myLineRenderer.SetPosition(1, gem.transform.position);
					myLineRenderer.SetWidth(0.1f, 0.1f);
				}
			}
		}
	}

	public void takeDmg(int dmg){
		life -= dmg;
	}

	public void EndIntro(){
		canAct = true;
		myCollider.enabled = true;
		nextFireball = Time.time + fireballDelay;
		nextIndio = Time.time + indioDelay;
	}
}
